/* $Id: LADSPAPluginDescriptor.hh,v 1.1 2003/08/07 08:57:08 andete Exp $

lemux - LADSPA instrument plugins based on openMSX devices

(c) 2003 Joost Yervante Damad <joost@lumatec.be>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef   __LADSPAPLUGINDESCRIPTOR_HH__
#define   __LADSPAPLUGINDESCRIPTOR_HH__

#include "ladspa.h"

#include <vector>
#include <string>

class LADSPAPortDescriptor;
class LADSPAPluginInstance;

class LADSPAPluginDescriptor: private LADSPA_Descriptor
{
public:
	LADSPAPluginDescriptor(
		unsigned long uniqueID,
		const std::string& label,
		const std::string& name,
		const std::string& maker,
		const std::string& copyright,
		LADSPA_Properties properties=0,
		void* implementationData=0
		);
	virtual ~LADSPAPluginDescriptor();

	const LADSPA_Descriptor* getLADSPA_Descriptor();

	// pure virtual factory function, called from the ladspa-C instantiate function
	virtual LADSPAPluginInstance* create(unsigned long SampleRate)=0;

	enum {
		RealTime      = LADSPA_PROPERTY_REALTIME,
		InplaceBroken = LADSPA_PROPERTY_INPLACE_BROKEN,
		HardRtCapable = LADSPA_PROPERTY_HARD_RT_CAPABLE,
	};

protected:
	
	// ports are deleted by LADSPAPluginDescriptor
	void addPort(LADSPAPortDescriptor* port);

private:
	std::vector<LADSPAPortDescriptor*> m_ports;
	void syncPorts();

	const std::string _label;
	const std::string _name;
	const std::string _maker;
	const std::string _copyright;
};

#endif // __LADSPAPLUGINDESCRIPTOR_HH__
