/* $Id: MUSICDrumInstance.hh,v 1.2 2003/08/08 12:05:13 andete Exp $

lemux - LADSPA instrument plugins based on openMSX devices

(c) 2003 Joost Yervante Damad <joost@lumatec.be>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef   __MUSICDrumINSTANCE_HH__
#define   __MUSICDrumINSTANCE_HH__

#include <math.h>

#include "LADSPAPluginInstance.hh"

typedef unsigned char byte;

class MUSICDrumInstance: public LADSPAPluginInstance
{
public:
	MUSICDrumInstance(unsigned long sampleRate);
	virtual ~MUSICDrumInstance();

	virtual void connect_port(unsigned long port, LADSPA_Data* dataLocation);
	virtual void activate();
	virtual void run(unsigned long SampleCount);

private:

	// ladspa ports:
	LADSPA_Data* port_freq_0;				//  0
	LADSPA_Data* port_freq_1;				//  1
	LADSPA_Data* port_freq_2;				//  2
	LADSPA_Data* port_amp_0;				//  3
	LADSPA_Data* port_amp_1;				//  4
	LADSPA_Data* port_amp_2;				//  5
	LADSPA_Data* port_output;				//  6
	LADSPA_Data* port_gate_bd;				//  7
	LADSPA_Data* port_gate_sd;				//  8
	LADSPA_Data* port_gate_tom;				//  9
	LADSPA_Data* port_gate_t_cy;			// 10
	LADSPA_Data* port_gate_hh;				// 11

	LADSPA_Data port_gate_bd_bak;
	LADSPA_Data port_gate_sd_bak;
	LADSPA_Data port_gate_tom_bak;
	LADSPA_Data port_gate_t_cy_bak;
	LADSPA_Data port_gate_hh_bak;

	byte rhythm_reg_copy;

	// constants
	const unsigned long sampleRate;

	// emulation control methods
	void reset();
	void setSampleRate();
	void writeReg(byte r, byte v);
	short maxVolume;

	class Slot
	{
	public:
		Slot();

		inline int volume_calc(byte LFO_AM);
		inline void KEY_ON (byte key_set);
		inline void KEY_OFF(byte key_clr);

		byte ar;	// attack rate: AR<<2
		byte dr;	// decay rate:  DR<<2
		byte rr;	// release rate:RR<<2
		byte KSR;	// key scale rate
		byte ksl;	// keyscale level
		byte ksr;	// key scale rate: kcode>>KSR
		byte mul;	// multiple: mul_tab[ML]

		// Phase Generator
		int phase;	// frequency counter
		int freq;	// frequency counter step
		byte fb_shift;	// feedback shift value
		int op1_out[2];	// slot1 output for feedback

		// Envelope Generator
		byte eg_type;	// percussive/nonpercussive mode
		byte state;	// phase type
		int TL;		// total level: TL << 2
		int TLL;	// adjusted now TL
		int volume;	// envelope counter
		int sl;		// sustain level: sl_tab[SL]

		byte eg_sh_dp;	// (dump state)
		byte eg_sel_dp;	// (dump state)
		byte eg_sh_ar;	// (attack state)
		byte eg_sel_ar;	// (attack state)
		byte eg_sh_dr;	// (decay state)
		byte eg_sel_dr;	// (decay state)
		byte eg_sh_rr;	// (release state for non-perc.)
		byte eg_sel_rr;	// (release state for non-perc.)
		byte eg_sh_rs;	// (release state for perc.mode)
		byte eg_sel_rs;	// (release state for perc.mode)

		byte key;	// 0 = KEY OFF, >0 = KEY ON

		// LFO
		byte AMmask;	// LFO Amplitude Modulation enable mask
		byte vib;	// LFO Phase Modulation enable flag (active high)

		int wavetable;	// waveform select
	};

	class Channel
	{
	public:
		Channel();
		inline int chan_calc(byte LFO_AM);
		inline void CALC_FCSLOT(Slot *slot);

		Slot slots[2];
		// phase generator state
		int block_fnum;	// block+fnum
		int fc;		// Freq. freqement base
		int ksl_base;	// KeyScaleLevel Base step
		byte kcode;	// key code (for key scaling)
		byte sus;	// sus on/off (release speed in percussive mode)
	};


	// internal methods:

	void init_tables();

	inline void advance_lfo();
	inline void advance();
	inline int rhythm_calc(bool noise);

	inline void set_mul(byte slot, byte v);
	inline void set_ksl_tl(byte chan, byte v);
	inline void set_ksl_wave_fb(byte chan, byte v);
	inline void set_ar_dr(byte slot, byte v);
	inline void set_sl_rr(byte slot, byte v);
	void load_instrument(byte chan, byte slot, byte* inst);
	void update_instrument_zero(byte r);
	void setRhythmMode(bool newMode);


	// internal var:
	Channel channels[9]; 
	// JYD 5/8/2003: but only use 1!

	byte instvol_r[9]; // // instrument/volume (or volume/volume in percussive mode)
	// JYD 5/8/2003: but only use 1!

	unsigned eg_cnt;		// global envelope generator counter
	unsigned eg_timer;		// global envelope generator counter works at frequency = chipclock/72
	unsigned eg_timer_add;	    	// step of eg_timer

	bool rhythm;			// Rhythm mode

	// LFO
	unsigned lfo_am_cnt;
	unsigned lfo_am_inc;
	unsigned lfo_pm_cnt;
	unsigned lfo_pm_inc;

	int noise_rng;		// 23 bit noise shift register
	int noise_p;		// current noise 'phase'
	int noise_f;		// current noise period

	// instrument settings
	//   0     - user instrument
	//   1-15  - fixed instruments
	//   16    - bass drum settings
	//   17-18 - other percussion instruments
	byte inst_tab[19][8];

	int fn_tab[1024];		// fnumber->increment counter

	byte LFO_AM;
	byte LFO_PM;
};

#endif // __MUSICDrumINSTANCE_HH__
