/* $Id: PSGDescriptor.cc,v 1.1 2003/08/05 12:57:56 andete Exp $

lemux - LADSPA instrument plugins based on openMSX devices

(c) 2003 Joost Yervante Damad <joost@lumatec.be>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "PSGDescriptor.hh"
#include "LADSPAPluginInstance.hh"
#include "LADSPAPortDescriptor.hh"
#include "LADSPAPluginDescriptor.hh"
#include "PSGInstance.hh"

#include <string>
#include <sstream>

PSGDescriptor::PSGDescriptor()
:
	LADSPAPluginDescriptor
		(
			2063,
			std::string("PSG"),
			std::string("PSG channel"),
			std::string("Joost Yervante Damad"),
			std::string("GPLv2")
		)
{
	LADSPAPortDescriptor *port;

	port = new LADSPAPortDescriptor
		(
			std::string("Freq (Hz)"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Audio ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::SampleRate   |
				LADSPAPortDescriptor::Logarithmic  |
				LADSPAPortDescriptor::Default440,

			0,
			0.5
		);
	addPort(port);

	port = new LADSPAPortDescriptor
		(
			std::string("Amplitude"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Control ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::Default0,

			0,
			15
		);
	addPort(port);
	
	port = new LADSPAPortDescriptor
		(
			std::string("Noise Control"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Control ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::Default0,

			0,
			1
		);
	addPort(port);

	port = new LADSPAPortDescriptor
	(
		std::string("Audio On"),
	
			LADSPAPortDescriptor::Input   |
			LADSPAPortDescriptor::Control ,
		
			LADSPAPortDescriptor::Default1|
			LADSPAPortDescriptor::Toggled ,

			0
		);
	addPort(port);

	port = new LADSPAPortDescriptor
	(
		std::string("Noise On"),
	
			LADSPAPortDescriptor::Input   |
			LADSPAPortDescriptor::Control ,
		
			LADSPAPortDescriptor::Default0|
			LADSPAPortDescriptor::Toggled ,

			0
		);
	addPort(port);

	port = new LADSPAPortDescriptor
	(
		std::string("AM On"),
	
			LADSPAPortDescriptor::Input   |
			LADSPAPortDescriptor::Control ,
		
			LADSPAPortDescriptor::Default0|
			LADSPAPortDescriptor::Toggled ,

			0
		);
	addPort(port);

	port = new LADSPAPortDescriptor
		(
			std::string("AM Freq (Hz)"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Control ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::Logarithmic  |
				LADSPAPortDescriptor::Default1,

			0.21336076380941482,
			13982.59765625
		);
	addPort(port);

	port = new LADSPAPortDescriptor
	(
		std::string("AM Env CONT"),
	
			LADSPAPortDescriptor::Input   |
			LADSPAPortDescriptor::Control ,
		
			LADSPAPortDescriptor::Default0|
			LADSPAPortDescriptor::Toggled ,

			0
		);
	addPort(port);

	port = new LADSPAPortDescriptor
	(
		std::string("AM Env ATT"),
	
			LADSPAPortDescriptor::Input   |
			LADSPAPortDescriptor::Control ,
		
			LADSPAPortDescriptor::Default0|
			LADSPAPortDescriptor::Toggled ,

			0
		);
	addPort(port);

	port = new LADSPAPortDescriptor
	(
		std::string("AM Env ALT"),
	
			LADSPAPortDescriptor::Input   |
			LADSPAPortDescriptor::Control ,
		
			LADSPAPortDescriptor::Default0|
			LADSPAPortDescriptor::Toggled ,

			0
		);
	addPort(port);

	port = new LADSPAPortDescriptor
	(
		std::string("AM Env HOLD"),
	
			LADSPAPortDescriptor::Input   |
			LADSPAPortDescriptor::Control ,
		
			LADSPAPortDescriptor::Default0|
			LADSPAPortDescriptor::Toggled ,

			0
		);
	addPort(port);

	port = new LADSPAPortDescriptor
		(
			std::string("Output"),
			
				LADSPAPortDescriptor::Output   |
				LADSPAPortDescriptor::Audio
		);
	addPort(port);

	port = new LADSPAPortDescriptor
		(
			std::string("Gate"),
			
				LADSPAPortDescriptor::Input   |
				LADSPAPortDescriptor::Audio ,
			
				LADSPAPortDescriptor::BoundedBelow |
				LADSPAPortDescriptor::BoundedAbove |
				LADSPAPortDescriptor::Default0,

			0,
			1
		);
	addPort(port);
}

PSGDescriptor::~PSGDescriptor()
{
}

LADSPAPluginInstance* PSGDescriptor::create(unsigned long SampleRate)
{
	return new PSGInstance(SampleRate);
}
